﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Analysis.SolutionCaching;
using HeuristicLab.Analysis.SolutionCaching.PermutationEncoding;
using HeuristicLab.Encodings.PermutationEncoding;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace AlgorithmBehaviorUnitTests {
  [TestClass]
  public class PermutationCacheTest {
    [TestMethod]
    public void TestPermutationSolutionDictionary() {
      var rand = new HeuristicLab.Random.FastRandom();
      PermutationEqualityComparer comp = new PermutationEqualityComparer();
      PermutationSolutionCache dict = new PermutationSolutionCache();
      int realDuplicates = 0;
      int noDuplicates = 0;

      Permutation[] permutations = new Permutation[500];
      for (int i = 0; i < permutations.Length; i += 2) {
        var p = new Permutation(PermutationTypes.RelativeUndirected, 21, rand);
        permutations[i] = p;
        permutations[i + 1] = p;

        PermutationSolutionInformation pi = new PermutationSolutionInformation();
        pi.Generation = rand.Next(0, 100);
        pi.ProducedBy = ProducedBy.Mutation;

        dict.Add(p, pi);
      }

      for (int i = 0; i < permutations.Length; i++) {
        for (int j = i + 1; j < permutations.Length; j++) {
          if (comp.Equals(permutations[i], permutations[j])) {
            realDuplicates++;
            break;
          }
        }
      }

      for (int i = 0; i < dict.SolutionDictionary.Keys.Count; i++) {
        for (int j = i + 1; j < dict.SolutionDictionary.Keys.Count; j++) {
          if (comp.Equals(dict.SolutionDictionary.Keys.ElementAt(i), dict.SolutionDictionary.Keys.ElementAt(j))) {
            noDuplicates++;
            break;
          }
        }
      }

      Assert.AreNotEqual(0, realDuplicates);
      Assert.AreEqual(0, noDuplicates);
      Assert.AreEqual(250, dict.SolutionDictionary.Keys.Count);
    }
  }
}
